"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mergeLhConfig = exports.getLhConfigFromArgv = exports.writeConfig = exports.readConfig = void 0;
const file_1 = require("../../../../core/file");
const loggin_1 = require("../../../../core/loggin");
const configPath_constant_1 = require("../../options/configPath.constant");
const budgets_1 = require("../../../assert/utils/budgets");
function readConfig(configPath = configPath_constant_1.DEFAULT_COLLECT_CONFIG_PATH) {
    const configJson = JSON.parse((0, file_1.readFile)(configPath, { fail: true }));
    return configJson;
}
exports.readConfig = readConfig;
function writeConfig(config, configPath = configPath_constant_1.DEFAULT_COLLECT_CONFIG_PATH) {
    (0, loggin_1.logVerbose)(`Update config under ${configPath}`);
    if (JSON.stringify(readConfig()) !== JSON.stringify(config)) {
        (0, file_1.writeFile)(configPath, JSON.stringify(config));
        (0, loggin_1.logVerbose)(`New config ${JSON.stringify(config)}`);
    }
    else {
        (0, loggin_1.logVerbose)(`No updates for ${configPath} to save.`);
    }
}
exports.writeConfig = writeConfig;
function getLhConfigFromArgv(rc) {
    let cfg = {};
    if (!!(rc === null || rc === void 0 ? void 0 : rc.configPath) && !!(rc === null || rc === void 0 ? void 0 : rc.config)) {
        throw new Error('configPath and config can\'t be used together');
    }
    if (rc === null || rc === void 0 ? void 0 : rc.configPath) {
        cfg = readConfig(rc.configPath);
        (0, loggin_1.logVerbose)(`LH Configuration ${rc.configPath} is used from CLI param or .user-flowrc.json`);
    }
    else if (rc === null || rc === void 0 ? void 0 : rc.config) {
        cfg = rc.config;
        (0, loggin_1.logVerbose)(`LH Configuration is used from config property .user-flowrc.json`);
    }
    if (!!(rc === null || rc === void 0 ? void 0 : rc.budgetPath) && !!(rc === null || rc === void 0 ? void 0 : rc.budgets)) {
        throw new Error('budgetPath and budgets can\'t be used together');
    }
    let budgets = undefined;
    if (rc === null || rc === void 0 ? void 0 : rc.budgetPath) {
        budgets = (0, budgets_1.readBudgets)(rc.budgetPath);
        (0, loggin_1.logVerbose)(`LH Performance Budget ${rc.budgetPath} is used from CLI param or .user-flowrc.json`);
    }
    else if (rc === null || rc === void 0 ? void 0 : rc.budgets) {
        budgets = rc.budgets;
        (0, loggin_1.logVerbose)(`LH Performance Budget is used from config property .user-flowrc.json`);
    }
    if (budgets) {
        cfg = Object.assign(Object.assign({}, cfg), { settings: Object.assign(Object.assign({}, cfg === null || cfg === void 0 ? void 0 : cfg.settings), { budgets }) });
    }
    return cfg;
}
exports.getLhConfigFromArgv = getLhConfigFromArgv;
function mergeLhConfig(globalCfg = {}, localCfg = {}) {
    var _a;
    let cfg = Object.assign({}, globalCfg);
    if (localCfg) {
        cfg = Object.assign(Object.assign(Object.assign({}, cfg), localCfg), { settings: Object.assign(Object.assign({}, cfg.settings), localCfg === null || localCfg === void 0 ? void 0 : localCfg.settings) });
        (0, loggin_1.logVerbose)(`LH Configuration is used from a user flow file`);
        if ((_a = localCfg === null || localCfg === void 0 ? void 0 : localCfg.settings) === null || _a === void 0 ? void 0 : _a.budgets) {
            (0, loggin_1.logVerbose)(`LH Performance Budget is used in a flows UserFlowProvider#flowOptions.settings.budgets`);
        }
    }
    if (Object.keys(cfg).length) {
        // Add extends if not given
        // @ts-ignore
        (cfg === null || cfg === void 0 ? void 0 : cfg.extends) || (cfg.extends = 'lighthouse:default');
    }
    return cfg;
}
exports.mergeLhConfig = mergeLhConfig;
//# sourceMappingURL=index.js.map